LBL<-function(dat, numSNPs, maxMissingGenos=1, allelic=TRUE, haplo.baseline = "missing", cov.baseline = "missing", interaction=T, a = 20, b = 20, start.beta = 0.01, lambda = 1, D = 0, seed = NULL, e = 0.1, burn.in = 20000, num.it = 50000)
{

    haplos.list<-pre.hapassoc(dat, numSNPs=numSNPs, pooling.tol=0, allelic=allelic, maxMissingGenos=maxMissingGenos)
    n.cov=dim(haplos.list$nonHaploDM)[2]-1#num of covariates

    ############################################################################################################
    #Case 1
    ############################################################################################################
    if(n.cov>=1){

    temp=cbind(haplos.list$ID, haplos.list$wt, haplos.list$nonHaploDM, haplos.list$haploDM)
    temp=temp[order(temp[,1]),]
    haplos.list$ID=temp[,1]
    haplos.list$wt=temp[,2]
    haplos.list$nonHaploDM=temp[,3:(3+n.cov)]
    haplos.list$haploDM=temp[(3+n.cov+1):ncol(temp)]

    if (is.null(seed)==FALSE) set.seed(seed)
    freq <- haplos.list$initFreq
    haplo.names <- names(freq)
    if (haplo.baseline=="missing") haplo.baseline <- haplo.names[which.max(freq)]
    column.subset <- haplo.names != haplo.baseline
    haplo.mat=haplos.list$haploDM[,column.subset]
    haplo.names=colnames(haplo.mat)

    freq.new<-freq[column.subset]
    freq.new[length(freq.new)+1]<-freq[haplo.baseline]

    if (cov.baseline=="missing") cov.baseline = rep(0, n.cov)
    haplos.list$nonHaploDM=as.matrix(haplos.list$nonHaploDM)   
    for(i in 1:n.cov)
    {
        cov.temp=dummy(haplos.list$nonHaploDM[,i+1])#transform the ith covariate into dummy variables
        cov.levels=sort(unique(haplos.list$nonHaploDM[,i+1]))
        cov.subset=(cov.levels!=cov.baseline[i])
        cov.temp=cov.temp[,cov.subset]#remove the baseline from dummy variables
        colnames.cov.temp=paste(colnames(haplos.list$nonHaploDM)[i+1], cov.levels[cov.subset], sep="")
        if(i==1)
        {
            cov.mat=cov.temp
            cov.names=colnames.cov.temp
        }else{
            cov.mat=cbind(cov.mat, cov.temp)#bind all the covariates into one matrix
            cov.names=c(cov.names, colnames.cov.temp)#bind all the covariate names into one matrix
        }
    }

    names=c(haplo.names, cov.names)

    y <- haplos.list$nonHaploDM[,1]
    N <- sum(haplos.list$wt)
    n=dim(haplo.mat)[1]
    num.haplo.id=as.vector(table(haplos.list$ID))

    h.length=ncol(haplo.mat)+1
    if(interaction==T){
        if (is.matrix(cov.mat)==T){x.length=ncol(haplo.mat)+ncol(cov.mat)+ncol(haplo.mat)*ncol(cov.mat)
        }else{x.length=ncol(haplo.mat)+1+ncol(haplo.mat)}
    }else{
        if (is.matrix(cov.mat)==T){x.length=ncol(haplo.mat)+ncol(cov.mat)
        }else{x.length=ncol(haplo.mat)+1}
    }

    haplo.map<-matrix(rep(NA,2*(dim(haplo.mat)[1])),ncol=2)
 	for(i in 1:dim(haplo.mat)[1])
	{
		for(j in 1:dim(haplo.mat)[2])
		{
			if(haplo.mat[i,j]==2)
			{
				haplo.map[i,1]=j
				haplo.map[i,2]=j
				break
			}
			if(haplo.mat[i,j]==1)
			{
				if(is.na(haplo.map[i,1])==T) haplo.map[i,1]=j
				else
				{
					haplo.map[i,2]=j
					break
				}
			}
		}
		if(is.na(haplo.map[i,1])==T)
		{
			haplo.map[i,1]=dim(haplo.mat)[2]+1
			haplo.map[i,2]=dim(haplo.mat)[2]+1
		}
		if(is.na(haplo.map[i,2])==T) haplo.map[i,2]=dim(haplo.mat)[2]+1
	}  
    
    uniq.mat<-unique(haplo.mat)
	uniq.map<-matrix(rep(NA,2*(dim(uniq.mat)[1])),ncol=2)
 	for(i in 1:dim(uniq.mat)[1])
	{
		for(j in 1:dim(uniq.mat)[2])
		{
			if(uniq.mat[i,j]==2)
			{
				uniq.map[i,1]=j
				uniq.map[i,2]=j
				break
			}
			if(uniq.mat[i,j]==1)
			{
				if(is.na(uniq.map[i,1])==T) uniq.map[i,1]=j
				else
				{
					uniq.map[i,2]=j
					break
				}
			}
		}
		if(is.na(uniq.map[i,1])==T)
		{
			uniq.map[i,1]=dim(uniq.mat)[2]+1
			uniq.map[i,2]=dim(uniq.mat)[2]+1
		}
		if(is.na(uniq.map[i,2])==T) uniq.map[i,2]=dim(uniq.mat)[2]+1
	}

    if (is.matrix(cov.mat)==T){cov.vec=apply(cov.mat, 1, function(x) paste(x, collapse=""))#concatenate each row of cov.mat
    }else{cov.vec=cov.mat}
    cov.vec.new=unique(cbind(haplos.list$ID, cov.vec))[,-1]
    y.new=unique(cbind(haplos.list$ID, y))[,-1]
    num.E=as.vector(table(cov.vec.new))#distribution of E's

    value.E=unique(cov.mat)#the dimention of value.E is len.E X len.dummy
    if (is.matrix(value.E)==T){
        value.E=value.E[order(unique(cov.vec)),]#reorder value.E to be consistant with num.E
        len.E=nrow(value.E)#num of different E's
        len.dummy=ncol(value.E)#num of dummy variables
    }else{
        value.E=value.E[order(unique(cov.vec))]#reorder value.E to be consistant with num.E
        len.E=length(value.E)#num of different E's
        len.dummy=1#num of dummy variables
    }

    if (is.matrix(value.E)==T){value.E.vec=apply(value.E, 1, function(x) paste(x, collapse=""))#concatenate each row of value.E
    }else{value.E.vec=value.E}
    index.E=match(cov.vec.new, value.E.vec)

    beta=rep(start.beta, x.length+1)
    beta.out<-numeric((num.it-burn.in)*(x.length+1))
    lambda.out<-numeric(num.it-burn.in)
    freq.out<-numeric((num.it-burn.in)*(h.length))
    D.out<-numeric(num.it-burn.in)

    if(interaction==F)no.interaction=1
    else no.interaction=0

    #dyn.load("mcmc.so")

    out<-.C("mcmc", as.integer(y.new), as.integer(N), as.integer(n), as.integer(num.haplo.id), as.integer(x.length), as.integer(h.length), as.integer(haplo.map), as.integer(dim(uniq.mat)[1]), as.integer(t(uniq.map)), as.integer(index.E), as.integer(t(value.E)), as.integer(num.E), as.integer(len.E), as.integer(len.dummy), as.double(beta), as.double(lambda), as.double(freq.new), as.double(D), as.double(a), as.double(b), beta.out=as.double(beta.out), lambda.out=as.double(lambda.out), freq.out=as.double(freq.out), D.out=as.double(D.out), as.integer(num.it), as.integer(burn.in), as.integer(no.interaction))
  
    beta.out<-matrix(out$beta.out,nrow=num.it-burn.in, byrow=TRUE)
    freq.out<-matrix(out$freq.out,nrow=num.it-burn.in, byrow=TRUE)

    post.mean.beta<-numeric(x.length+1)
    ci.beta<-numeric((x.length+1)*2)
    post.mean.freq<-numeric(h.length)
    ci.freq<-numeric((h.length)*2)
    ci.lambda<-numeric(2)
    ci.D<-numeric(2)
 
    k<-1
    for (i in 1:(x.length+1))
      {
        ci.beta[k]<-quantile(beta.out[,i], probs=0.025)
        ci.beta[k+1]<-quantile(beta.out[,i], probs=0.975)
        k<-k+2
        post.mean.beta[i]<-mean(beta.out[,i])
      }
    ci.beta<-matrix(ci.beta,nrow=x.length+1, ncol=2, byrow=TRUE)
    ci.OR<-data.frame(exp(ci.beta))
    OR<-exp(post.mean.beta)
    
    k<-1
    for (i in 1:(h.length))
      {
        ci.freq[k]<-quantile(freq.out[,i], probs=0.025)
        ci.freq[k+1]<-quantile(freq.out[,i], probs=0.975)
        k<-k+2
        post.mean.freq[i]<- mean(freq.out[,i])
      }

    ci.lambda<-c(quantile(out$lambda.out, probs=0.025),quantile(out$lambda.out, probs=0.975))
    ci.D<-c(quantile(out$D.out, probs=0.025),quantile(out$D.out, probs=0.975))

    prob.alt<-numeric(x.length+1)
    BF<-numeric(x.length+1)
    
    for (i in 1:(x.length+1))
      {
        prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
      }

    for (i in 1:(x.length+1))
      {
        prior.prob<-(b/(e+b))^a
        if (prob.alt[i]<1) 
          {
            bf<-(prob.alt[i]/(1-prob.alt[i]))/(prior.prob/(1-prior.prob))
            BF[i]<-round(bf,4)
          } else
        BF[i]<-">100"
      }

    OR<-round(OR,4)
    ci.OR<-round(ci.OR,4)
    post.mean.freq<-round(post.mean.freq,4)
    ci.freq<-round(ci.freq,4)
    ci.lambda<-round(ci.lambda,4)
    ci.D<-round(ci.D,4)    

    if(interaction==T){
        for(i in 1:len.dummy)
        {
            name.paste.temp<-paste(cov.names[i],haplo.names,sep=".")
            if(i==1){name.paste=name.paste.temp
            }else{name.paste=c(name.paste, name.paste.temp)}
        }
        pname<-c("beta0",haplo.names,name.paste,cov.names)
    }else{
        pname<-c("beta0",haplo.names,cov.names)
    }

    names(BF)<-pname
    names(OR)<-pname

    ci.OR<-data.frame(pname,ci.OR)
    colnames(ci.OR)<-c("Name","Lower","Upper")

    ci.freq<-matrix(ci.freq,nrow=(h.length),ncol=2,byrow=TRUE)
    ci.freq<-data.frame(c(haplo.names,haplo.baseline),ci.freq)
    colnames(ci.freq)<-c("Name","Lower","Upper")
    names(post.mean.freq)<-c(haplo.names,haplo.baseline)

    ans <- list(BF=BF, OR=OR, CI.OR=ci.OR, freq=post.mean.freq, CI.freq=ci.freq, CI.lambda=ci.lambda, CI.D=ci.D)
    }

    ############################################################################################################
    #Case 2
    ############################################################################################################
    if(n.cov==0){

    temp=cbind(haplos.list$ID, haplos.list$wt, haplos.list$nonHaploDM, haplos.list$haploDM)
    temp=temp[order(temp[,1]),]
    haplos.list$ID=temp[,1]
    haplos.list$wt=temp[,2]
    haplos.list$nonHaploDM=temp[,3]
    haplos.list$haploDM=temp[4:ncol(temp)]

    if (is.null(seed)==FALSE) set.seed(seed)
    haplo.names <- names(haplos.list$initFreq)
    freq <- haplos.list$initFreq
    if (haplo.baseline=="missing")haplo.baseline <- haplo.names[which.max(freq)]
    column.subset <- colnames(haplos.list$haploDM) != haplo.baseline  
    hdat <- cbind(haplos.list$nonHaploDM, haplos.list$haploDM[, column.subset])
    colnames(hdat) <- c("affected", colnames(haplos.list$haploDM[, column.subset]))
    ID <- haplos.list$ID
    N <- sum(haplos.list$wt)
    y <- as.numeric(hdat[, 1])# y <- as.numeric(hdat[colnames(hdat)!="pheno"])
    x <- data.matrix(hdat[, -1])# x <- data.matrix(hdat[colnames(hdat)!="pheno"])
    colnames(x)<-NULL

    freq.new<-freq[names(haplos.list$haploDM[, column.subset])]
    freq.new[length(freq.new)+1]<-freq[haplo.baseline]
    freq.new<-as.vector(freq.new)
    num.haplo.id<-as.vector(table(ID))
    x.length<-as.integer(dim(x)[2])
    unique.x<-unique(x)	

    beta=rep(start.beta, x.length)
    beta.out<-numeric((num.it-burn.in)*x.length)
    lambda.out<-numeric(num.it-burn.in)
    freq.out<-numeric((num.it-burn.in)*(x.length+1))
    D.out<-numeric(num.it-burn.in)

    #dyn.load("BL.so")
    
    out<-.C("mcmc_without_cov", as.integer(x), as.integer(dim(x)[1]), as.integer(y), as.integer(N), as.integer(num.haplo.id), as.integer(x.length), as.double(freq.new), as.double(D), as.double(beta), as.double(a), as.double(b), as.integer(t(unique.x)), as.integer(dim(unique.x)[1]), as.double(lambda), as.integer(num.it), as.integer(burn.in), beta.out=as.double(beta.out), lambda.out=as.double(lambda.out), freq.out=as.double(freq.out), D.out=as.double(D.out))

    beta.out<-matrix(out$beta.out,nrow=num.it-burn.in, byrow=TRUE)
    freq.out<-matrix(out$freq.out,nrow=num.it-burn.in, byrow=TRUE)

    ci.beta<-numeric(x.length*2)
    ci.lambda<-numeric(2)
    ci.D<-numeric(2)
    post.mean.beta<-numeric(x.length)
    ci.freq<-numeric((x.length+1)*2)
    post.mean.freq<-numeric(x.length+1) 
    k<-1
    for (i in 1:x.length)
      {
        ci.beta[k]<-quantile(beta.out[,i], probs=0.025)
        ci.beta[k+1]<-quantile(beta.out[,i], probs=0.975)
        k<-k+2
        post.mean.beta[i]<- mean(beta.out[,i])
      }
    
    ci.lambda<-c(quantile(out$lambda.out, probs=0.025),quantile(out$lambda.out, probs=0.975))
    ci.D<-c(quantile(out$D.out, probs=0.025),quantile(out$D.out, probs=0.975))

    k<-1
    for (i in 1:(x.length+1))
      {
        ci.freq[k]<-quantile(freq.out[,i], probs=0.025)
        ci.freq[k+1]<-quantile(freq.out[,i], probs=0.975)
        k<-k+2
	post.mean.freq[i]<- mean(freq.out[,i])
      } 

    prob.alt<-numeric(x.length)
    BF<-numeric(x.length)
    
    for (i in 1:x.length)
      {
        prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
      }

    for (i in 1:x.length)
      {
        prior.prob<-(b/(e+b))^a
        if (prob.alt[i]<1) 
          {
            BF[i]<-(prob.alt[i]/(1-prob.alt[i]))/(prior.prob/(1-prior.prob))
            #BF[i]<-round(BF[i],8)
          } else
        BF[i]<-">100"
      }

    names(BF)<-colnames(hdat[, -1])
    names(post.mean.beta)<-colnames(hdat[, -1])

    ci.beta<-matrix(ci.beta,nrow=x.length, ncol=2, byrow=TRUE)
    ci.beta<-data.frame(colnames(hdat[, -1]),exp(ci.beta))
    colnames(ci.beta)<-c("Hap", "Lower", "Upper")

    ci.freq<-matrix(ci.freq,nrow=(x.length+1), ncol=2, byrow=TRUE)
    ci.freq<-data.frame(c(colnames(hdat[, -1]),haplo.baseline), ci.freq)
    colnames(ci.freq)<-c("Hap", "Lower", "Upper")
	names(post.mean.freq)<-c(colnames(hdat[, -1]), haplo.baseline)
    ans <- list(BF = BF, OR = exp(post.mean.beta), CI.OR = ci.beta, freq=post.mean.freq, CI.freq = ci.freq, CI.lambda = ci.lambda, CI.D = ci.D)
    }

    #############################################################
    return (ans)
}

